<?php

/**
 * Copyright (C) 2020 Tencent Cloud.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

namespace App\Api\Serializer;

use App\Models\Group;
use Discuz\Api\Serializer\AbstractSerializer;
use Tobscure\JsonApi\Relationship;

class GroupSerializer extends AbstractSerializer
{
    /**
     * {@inheritdoc}
     */
    protected $type = 'groups';

    /**
     * {@inheritdoc}
     *
     * @param Group $model
     * @return array
     */
    public function getDefaultAttributes($model)
    {
        return [
            'name'              => $model->name,
            'type'              => $model->type,
            'color'             => $model->color,
            'icon'              => $model->icon,
            'default'           => $model->default,
            'isDisplay'         => (bool) $model->is_display,
            'isPaid'            => (bool) $model->is_paid,
            'fee'               => (float) $model->fee,
            'days'              => (int) $model->days,
            'scale'             => $model->scale,
            'is_subordinate'    => (bool) $model->is_subordinate,
            'is_commission'     => (bool) $model->is_commission,
        ];
    }

    /**
     * @param $group
     * @return Relationship
     */
    public function permission($group)
    {
        return $this->hasMany($group, GroupPermissionSerializer::class);
    }

    public function permissionWithoutCategories($group)
    {
        return $this->hasMany($group, GroupPermissionSerializer::class);
    }
}
